package geniusweb.simplerunner.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.TextField;
import java.net.URISyntaxException;
import java.util.Arrays;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.deadline.Deadline;
import geniusweb.deadline.DeadlineRounds;
import geniusweb.deadline.DeadlineTime;
import geniusweb.protocol.NegoSettings;
import geniusweb.protocol.session.TeamInfo;
import geniusweb.protocol.session.saop.SAOPSettings;
import geniusweb.references.Parameters;
import geniusweb.references.PartyRef;
import geniusweb.references.PartyWithParameters;
import geniusweb.references.PartyWithProfile;
import geniusweb.references.ProfileRef;

public class SettingsPanel extends JPanel {
	private final ObjectMapper jackson = new ObjectMapper();
	private final JComboBox<String> protocolcombo = new JComboBox<>(
			new String[] { "SAOP" });

	private final TimePanel timepanel = new TimePanel();
	private final TextField nextparty = new TextField(30);
	private final TextField nextparameters = new TextField(30);
	private final MyFileChooser nextprofile = new MyFileChooser();
	private final JButton addButton = new JButton("add");
	private final SelectionModel selectedparties = new SelectionModel();

	public SettingsPanel() {
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(new WithLabel("protocol", protocolcombo));
		add(new WithLabel("deadline", timepanel));
		add(new JSeparator());
		add(new JLabel("participants"));
		add(new WithLabel("party:", nextparty));
		add(new WithLabel("parameters:", nextparameters));
		add(new WithLabel("profile:", nextprofile));
		add(new JSeparator());
		add(addButton);
		add(new JLabel("Selected Profiles, parties for the session"));
		add(new JScrollPane(new JTable(selectedparties)));
		addButton.addActionListener(evt -> addparty());
	}

	private void addparty() {
		try {
			addparty1();
		} catch (IllegalArgumentException e) {
			JScrollPane message = new JScrollPane(
					new JTextArea(e.getMessage(), 10, 40));
			JOptionPane.showMessageDialog(this, message, "Failed to add party",
					JOptionPane.WARNING_MESSAGE, null);
		}
	}

	private void addparty1() {
		PartyRef partyref;
		try {
			partyref = new PartyRef("classpath:" + nextparty.getText());
		} catch (URISyntaxException e) {
			throw new IllegalArgumentException("Party not set correctly:" + e);
		}
		Parameters parameters;
		try {
			parameters = jackson.readValue("{" + nextparameters.getText() + "}",
					Parameters.class);
		} catch (JsonProcessingException e) {
			throw new IllegalArgumentException(
					"Parameters not set correctly:" + e);
		}
		ProfileRef profile;
		try {
			profile = new ProfileRef("file:" + nextprofile.getFile());
		} catch (URISyntaxException e) {
			throw new IllegalArgumentException(
					"Profile not set correctly:" + e);
		}
		PartyWithParameters partywithparams = new PartyWithParameters(partyref,
				parameters);
		PartyWithProfile partywithprofile = new PartyWithProfile(
				partywithparams, profile);
		TeamInfo teaminfo = new TeamInfo(Arrays.asList(partywithprofile));
		selectedparties.addTeam(teaminfo);
	}

	public NegoSettings getSettings() {
		return new SAOPSettings(selectedparties.getTeams(),
				timepanel.getDeadline());
	}

}

/**
 * adds label to left of component
 */
class WithLabel extends JPanel {
	public WithLabel(String label, Component component) {
		setLayout(new BorderLayout());
		add(new JLabel(label), BorderLayout.WEST);
		add(component, BorderLayout.CENTER);
	}
}

class TimePanel extends JPanel {
	private static final long DEFAULTDEADLINE = 10000;
	JComboBox<String> unitcombo = new JComboBox<String>(
			new String[] { "seconds", "rounds" });
	private SpinnerModel time = new SpinnerNumberModel(10, 1, 1000, 1);

	public TimePanel() {
		setLayout(new BorderLayout());
		add(new JSpinner(time), BorderLayout.CENTER);
		add(unitcombo, BorderLayout.EAST);
	}

	public Deadline getDeadline() {
		if (unitcombo.getSelectedIndex() == 0)
			return new DeadlineTime((int) time.getValue() * 1000);
		return new DeadlineRounds((int) time.getValue(), DEFAULTDEADLINE);
	}
}